/* The TTP Project Corncob Dynamic Kernel Loader
   Version 0.1.1
   Copyright 2001-2002 Leif Ames and Michael Benoit
   This file is placed under the "Artistic License", you may
   redisribute and/or modify it as per the terms of that license
*/

#include "gdt.h"
void _start(void)
{
	main();                   //Call to main()
}
int position;
void int_newint();
void int_smileyscreen();
void int_printchr();
extern void _newint(void);
extern void _smileyscreen(void);
extern void _printchr(void);

char *vidmem = (char *) 0xb8000;

desc_table(IDT, 256) { };

struct {
   unsigned short limit __attribute__ ((packed));
   union DT_entry *idt __attribute__ ((packed));
   } loadidt = { (256 * sizeof(union DT_entry) - 1), IDT};

void set_vector(void *handler, unsigned char interrupt, unsigned short control_major) {
   unsigned short codesegment = 0x08;
   asm volatile("movw %%cs,%0":"=g" (codesegment));

   IDT[interrupt].gate.offset_low = (unsigned short) (((unsigned long)handler)&0xffff);
   IDT[interrupt].gate.selector = codesegment;
   IDT[interrupt].gate.access = control_major;
   IDT[interrupt].gate.offset_high = (unsigned short) (((unsigned long)handler) >> 16);
   }

void int_null();
asm (
   ".globl int_null	\n"
   "int_null:		\n"
   "   iret		\n"  /* Exit Interrupt */
   );

void InitIDT() {
   int cnt;
   for(cnt=0;cnt<32;cnt++) {
      set_vector(int_null, cnt, D_PRESENT + D_INT + D_DPL1);
      }
   for(;cnt<256;cnt++) {
      set_vector(int_null, cnt, D_PRESENT + D_INT + D_DPL3);
      }

   set_vector(_newint, 33, D_PRESENT + D_INT + D_DPL3);
   set_vector(_printchr, 35, D_PRESENT + D_INT + D_DPL3);

   asm (
      "lidt (%0)			\n"  /* Load the IDT */
      "pushfl				\n"  /* Clear the NT flag */
      "andl $0xffffbfff, (%%esp)	\n"
      "popfl				\n"
      :
      : "r" ((char *) &loadidt)
      );
   asm("sti");
   }
asm("
	.globl _newint	\n
	.align 4	\n
	_newint:	\n
		call int_newint	\n
		iret			\n");
void int_newint(void)
{
	long int newintaddr;
	int newintnum;
	asm("
		movl %%eax,%0\n
		mov %%bx,%1\n"
		: "=g" (newintaddr), "=g" (newintnum));
	set_vector((int *)newintaddr,newintnum, D_PRESENT + D_INT + D_DPL3);
}
asm("
	.globl _smileyscreen	\n
	.align 4		\n
	_smileyscreen:		\n
		call int_smileyscreen	\n
		iret			\n");
void int_smileyscreen() {
   int cnt;
   for(cnt=0;cnt<80*25;cnt++) {
      vidmem[cnt*2] = 0x02;
      vidmem[cnt*2+1] = 0x07;
      }
}
asm("
	.globl _printchr	\n
	.align 4		\n
	_printchr:		\n
		pushl %cx		\n
		call int_printchr	\n
		popl %cx 			\n
		iret			\n");
void int_printchr() {
   int chr;
   asm("mov %%ax,%0\n"
   	: "=g" (chr) );
   vidmem[position*2] = chr;
   vidmem[position*2+1] = 0x07;
   position++;	
   if(chr==0x07)
   {
	int times;
	times=position/80;
	position=position-(80*times);
	position=(80-position)+(80*times);
   }
 }

int main() {
   int cnt;
   InitIDT();
   for(cnt=0;cnt<80*25;cnt++) {
      vidmem[cnt*2] = 0x01;
      vidmem[cnt*2+1] = 0x07;
      }
   asm("
      movl %0,%%eax\n
      mov $34,%%bx\n"
      : :  "g" (&_smileyscreen) );
   asm("int $33\n");
   asm("ljmp $0x08,$0x21000\n");                  //not the devil anymore
}
